import { Request, Response } from "express";
import { database } from "../services/database";
import * as http from 'typed-rest-client/HttpClient';
import { jwt } from "../services/passport";
import { OAuthData } from "../../data/oauth";
import { v4 as uuidv4 } from 'uuid';
import moment from "moment";
const pino = require('pino-http')();


export async function connectionPreparation(req: Request, res: Response) {
  const state = req.query['state'];
  const access_token = req.query['token'];
  let expires_in = req.query['expires_in'];
  if (!state || !access_token) {
    res.status(400).send({ error: 'Missing fields in the body.' });
    return;
  }

  const connection = await database.oneOrNone('SELECT "name", "type", "clientId", "grantType", "userId" FROM "ConnectionState" WHERE "state" = $1', [state]);
  if (!connection) {
    res.status(400).send({ error: 'Invalid combination.' });
    return;
  }

  if (connection.type == 'twitch') {
    const rest = new http.HttpClient(null);
    const response = await rest.get('https://id.twitch.tv/oauth2/validate', {
      'Authorization': 'OAuth ' + access_token,
    });
    const body = await response.readBody();
    const json = JSON.parse(body);
    expires_in = json.expires_in;

    if (!expires_in) {
      res.status(400).send({ error: 'Twitch API is not working correctly.' });
      return;
    }
  }

  const expiration = expires_in ? parseInt(expires_in.toString()) : null;
  if (!expiration || isNaN(expiration) || !isFinite(expiration) || expiration < 0) {
    res.status(400).send({ error: 'Could not determine the expiration of the token.' });
    return;
  }

  const expires_at = moment().add(expiration - 300, 's');
  res.send({
    data: { connection, expires_at },
  });
};

export async function connectionCallback(req: any, res: Response) {
  const name = req.body.name;
  const type = req.body.type?.toLowerCase();
  const client_id = req.body.client_id;
  const grant_type = req.body.grant_type?.toLowerCase();
  if (!name || !type || !client_id || !grant_type) {
    res.status(400).send({ error: 'Missing fields in the body.' });
    return;
  }

  const url = OAuthData[type].endpoint;
  const redirect = OAuthData[type].redirect;
  const scopes = OAuthData[type].scopes.join(' ');
  const nounce = uuidv4();
  await database.none('INSERT INTO "ConnectionState" ("name", "type", "clientId", "grantType", "state", "userId") VALUES ($1, $2, $3, $4, $5, $6)'
    + ' ON CONFLICT ("userId", "name") DO UPDATE SET "type" = $2, "clientId" = $3, "grantType" = $4, "state" = $5;',
    [name, type, client_id, grant_type, nounce, req.user.id]);

  const redirect_uri = url + '?client_id=' + client_id + '&force_verify=true&redirect_uri=' + redirect + '&response_type=token&scope=' + scopes + '&state=' + nounce;
  res.send({ success: true, error: null, data: redirect_uri });
};

// login/register
export async function twitchCallback(req: any, res: any) {
  const query = `client_id=${process.env.AUTH_CLIENT_ID}&client_secret=${process.env.AUTH_CLIENT_SECRET}&code=${req.body.code}&grant_type=authorization_code&redirect_uri=${process.env.AUTH_REDIRECT_URI}`
  const rest = new http.HttpClient(null);
  const response = await rest.post('https://id.twitch.tv/oauth2/token', query, {
    'Content-Type': 'application/x-www-form-urlencoded'
  });
  const body = await response.readBody();
  const codeData = JSON.parse(body);
  if (!codeData || codeData.message) {
    console.log('Failed to validate Twitch code authentication:', codeData);
    res.send({ authenticated: false });
    return;
  }
  console.log('Validated Twitch code authentication:', codeData);

  const resp = await rest.get('https://api.twitch.tv/helix/users', {
    'Authorization': 'Bearer ' + codeData.access_token,
    'Client-Id': process.env.AUTH_CLIENT_ID
  });
  const b = await resp.readBody();
  const userData = JSON.parse(b);
  if (!userData?.data) {
    console.log('Failed to fetch twitch data:', codeData, userData?.data);
    res.send({ authenticated: false, error: 'Twitch API is not working correctly.' });
    return;
  }
  console.log('Fetched Twitch user data:', userData);

  const account: any = await database.oneOrNone('SELECT "userId" FROM "Account" WHERE "providerAccountId" = $1', userData.data[0].id);
  if (account != null) {
    const user: any = await database.one('SELECT id FROM "User" WHERE id = $1', account.userId);
    const token = jwt.sign({ id: user.id }, process.env.JWT_SECRET, { expiresIn: '30d' });
    res.send({ authenticated: true, token: token });

    var now = Date.now();
    const expires_at = ((now / 1000) | 0) + codeData.expires_in;
    await database.none('UPDATE "Account" SET refresh_token = COALESCE($1, refresh_token), access_token = $2, id_token = COALESCE($3, id_token), expires_at = $4, scope = $5 WHERE "userId" = $6', [codeData.refresh_token, codeData.access_token, codeData.id_token, expires_at, codeData.scope.join(' '), account.userId]);
    return;
  }

  res.send({ authenticated: false });
};

export async function validate(req: any, res: Response, next: () => void) {
  res.send({ authenticated: req?.user != null, user: req?.user });
}